#-----------------------------------------------------------------------
# Aplicação de Jackknife em modelos de regressão não linear.

library(latticeExtra)
library(car)
library(alr3)

str(segreg)
help(segreg, help_type = "html")

xyplot(C ~ Temp, data = segreg, type = c("p", "smooth"))

# Curva "palpite".
start <- list(th0 = 75, th1 = 0.5, th2 = 50)
xyplot(C ~ Temp, data = segreg) +
    layer(panel.curve(th0 + th1 * (x - th2) * (x >= th2) +
                      0 * (x < th2), lty = 2),
          data = start)

# Ajuste.
n0 <- nls(C ~ th0 + th1 * (Temp - th2) * (Temp >= th2) +
              0 * (Temp < th2),
          data = segreg,
          start = start)

# Estimativas e medidas de ajuste.
summary(n0)

# Intervalos de confiança.
# confint(n2)
confint.default(n0)

# Observados e preditos.
xyplot(C ~ Temp, data = segreg) +
    layer(panel.curve(th0 + th1 * (x - th2) * (x >= th2) +
                      0 * (x < th2)),
          data = as.list(coef(n0)))

#-----------------------------------------------------------------------
# Estimativas Jackknife para os parâmetros.

n <- nrow(segreg)
theta <- coef(n0)

thetaj <- matrix(0, nrow = n, ncol = length(theta))
colnames(thetaj) <- names(theta)

# Ajustes deixando uma observação de fora (leave-one-out).
for (i in 1:n) {
    # Aqui estima com leave-one-out.
    n1 <- nls(C ~ th0 + th1 * (Temp - th2) * (Temp >= th2) +
                  0 * (Temp < th2),
              data = segreg[-i, ],
              start = coef(n0))
    # Aqui calcula-se a estimativa Jackknife.
    thetaj[i, ] <- n * theta - (n - 1) * coef(n1)
}

thetaj
pairs(thetaj)

# Estimativas pontuais.
cbind(Jack = colMeans(thetaj),
      MLE = theta)

# Erros padrões.
cbind(Jack = apply(thetaj, MARGIN = 2, sd)/sqrt(n),
      MLE = summary(n0)$coefficients[, "Std. Error"])

# Pacote com função pronta para Jackknife para modelos não lineares.
library(nlstools)
ls("package:nlstools")

# Aplica Jackknife sobre o modelo.
j0 <- nlsJack(n0)
summary(j0)

#-----------------------------------------------------------------------
# Aplicação na Mediana (função "discretas") -> problema.

# Ordena o vetor de valores.
x <- sort(precip)

# Calcula a mediana com os dados originais.
M <- median(x)
M

n <- length(x)
i <- 1:length(x)

# Estimativas da mediana por Jackknife.
y <- sapply(i,
            FUN = function(i) {
                ep <- median(x[-i])
                pv <- n * M - (n - 1) * ep
                return(pv)
            })

stem(y)

# NOTE: Explique porque o Jackknife para a mediana só retornou 2
# valores? Considere que o tamanho da amostra é dado abaixo.
length(x)

#-----------------------------------------------------------------------
# Capacidade de predição de modelos.

str(cars)
plot(dist ~ speed, cars)

i <- 1:nrow(cars)

# As funções interno() e externo() retornam as correlações entre predito
# e observado tendo a observação predita dentro da amostra (interno) ou
# fora (leave-one-out).

externo <- function(grau) {
    Y <- sapply(i,
                FUN = function(i) {
                    m0 <- lm(dist ~ poly(speed, grau),
                             data = cars[-i, ])
                    yi <- predict(m0, newdata = cars[i, ])
                    return(cbind(yi, cars[i, ]$dist))
                })
    # R² do observado com o predito leave-one-out.
    cor(Y[1, ], Y[2, ])^2
}

interno <- function(grau) {
    # Ajuste do modelo com todos os dados.
    m0 <- lm(dist ~ poly(speed, grau), data = cars)
    summary(m0)$r.squared
}

g <- 1:8
int <- sapply(g, interno)
ext <- sapply(g, externo)

xyplot(int + ext ~ g,
       type = "o",
       auto.key = TRUE,
       xlab = "Grau do polinômio",
       ylab = "Coeficiente de determinação")

#-----------------------------------------------------------------------
# Inferência para a DL 50.

library(labestData)

data(PaulaTb3.12)
str(PaulaTb3.12)
help(PaulaTb3.12, h = "html")

# Converte a resposta para binário.
PaulaTb3.12$resp <- as.integer(PaulaTb3.12$resp) - 1

plot(resp ~ vol, data = PaulaTb3.12)

m0 <- glm(resp ~ vol,
          data = PaulaTb3.12,
          family = binomial)
summary(m0)

# DL_50.
dl <- -coef(m0)[1]/coef(m0)[2]

# str(m0$family)$link

plot(resp ~ vol, data = PaulaTb3.12)
curve(m0$family$linkinv(coef(m0)[1] + coef(m0)[2] * x),
      add = TRUE)
abline(v = dl, h = 0.5, lty = 2)

n <- nrow(PaulaTb3.12)
i <- 1:n

# Estimativas por Jackknife.
DL <- sapply(i,
             FUN = function(i) {
                 m0 <- glm(resp ~ vol,
                           data = PaulaTb3.12[-i, ],
                           family = binomial)
                 # Estimativa Parcial.
                 DL <- -coef(m0)[1]/coef(m0)[2]
                 # Pseudo-valor.
                 n * dl - (n - 1) * DL
             })

m <- mean(DL)
e <- sd(DL)/sqrt(n)

plot(density(DL))
rug(DL)

# NOTE: alguma pista sobre porque a distribuição fica bimodal?

#-----------------------------------------------------------------------
# Tempo do primeiro lugar nas provas de Monza desde 1950.

# Lendo tabela direto do site.
da <- XML::readHTMLTable("http://www.4mula1.ro/s/?cdx=9b0870e&pos=1&idt=44")
da <- da[[1]]

summary(da)

# Anos das corridas.
da$ano <- as.integer(sub("^(\\d{4})-.*$", "\\1", da$Date))

# Tempo do primeiro colocado na prova.
library(lubridate)

x <- hms(da$Time)
da$tpro <- 60 * hour(x) + minute(x) + second(x)/60

#-----------------------------------------------------------------------
# browseURL("http://www.4mula1.ro/statistics/grandprix/")

xyplot(tpro ~ ano, data = da, type = c("p", "smooth"))

# Cria uma variável que começa em 0.
da$anoc <- da$ano - min(da$ano)
da <- na.omit(da)

plot(tpro ~ anoc, data = da)
start <- list(b0 = 160, b1 = -3, xb = 25)
with(start, {
    curve(b0 + b1 * x * (x <= xb) + b1 * xb * (x > xb),
          add = TRUE, col = 2)
})

# Ajuste do modelo não linear.
n0 <- nls(tpro ~ b0 +
              b1 * anoc * (anoc <= xb) +
              xb * b1 * (anoc > xb),
          data = da, start = start)

# Resumo do ajuste.
summary(n0)

# Intervalos de confiança.
# confint(n0)
confint.default(n0)

# Curva ajustada.
plot(tpro ~ anoc, data = da)
with(as.list(coef(n0)), {
    curve(b0 + b1 * x * (x <= xb) + b1 * xb * (x > xb),
          add = TRUE,
          col = 4)
})

#-----------------------------------------------------------------------
# Estimativas Jackknife para os parâmetros.

n <- nrow(da)
theta <- coef(n0)

thetaj <- matrix(0, nrow = n, ncol = length(theta))
colnames(thetaj) <- names(theta)

for (i in 1:n) {
    n1 <- nls(tpro ~ b0 +
                  b1 * anoc * (anoc <= xb) +
                  xb * b1 * (anoc > xb),
              data = da[-i, ],
              start = coef(n0))
    thetaj[i, ] <- n * theta - (n - 1) * coef(n1)
}

thetaj

# pairs(M)
pairs(thetaj)

jktheta <- colMeans(thetaj)
cbind(theta, jktheta)

jkse <- apply(thetaj, MARGIN = 2, sd)/sqrt(n)
cbind(se = summary(n0)$coefficients[, 2], jkse = jkse)

jktheta + pnorm(0.975) * jkse

jkic <- sweep(outer(jkse,
                    c(fit = 0, lwr = -1, upr = 1) * pnorm(0.975),
                    FUN = "*"),
              MARGIN = 1, STATS = jktheta, FUN = "+")
jkic

#-----------------------------------------------------------------------

library(nlstools)
ls("package:nlstools")

j0 <- nlsJack(n0)
summary(j0)

help(nlsJack, h = "html")

# Ainda não sei reproduzir o IC calculado pela nlsJack mas está descrito
# em Seber & Wild (1989).

#-----------------------------------------------------------------------
